from django.core.management.base import BaseCommand, CommandError
from documents.models import Image
import gzip
import os
import re
import logging

logging.basicConfig(
    level=logging.INFO,
    format='[%(levelname)s] %(message)s',
)
logger = logging.getLogger(__name__)

REGEX_INDEX = re.compile(b'^line_(\d+) (.+) \d+ ([\de\-\.]+) (\d+) (\d+) (\d+) (\d+)')

class Command(BaseCommand):
    help = 'Import indexes into ElasticSearch'

    def add_arguments(self, parser):
        parser.add_argument(
            'files',
            nargs='+',
            help='Indexes source files'
        )
        parser.add_argument(
            '--iiif-base-url',
            type=str,
            help='IIIF base url where to find images',
            required=True,
        )

    def handle(self, *args, **options):
        for path in options['files']:
            self.import_indexes(path, options['iiif_base_url'])

    def import_indexes(self, index_path, iiif_base_url, extension='jpg'):
        """
        Import indexes from file
        One gzipped index file per image
        Format: #line word n/a score-[0,1] x1 y1 x2 y2
        """
        assert os.path.exists(index_path)
        assert index_path.endswith('.idx.gz')

        # Find image reference in database
        filename = os.path.basename(index_path)[:-7]
        url = '{}/{}.{}'.format(
            iiif_base_url,
            filename,
            extension,
        )
        image, _ = Image.objects.get_or_create(iiif_url=url)
        logger.info('Image {}'.format(image))

        # Import each index
        with gzip.open(index_path) as f:
            for line in f.readlines():
                if line.decode('utf-8').startswith('#'):
                    continue
                index = REGEX_INDEX.match(line)
                if index is None:
                    logger.warn('Index parsing failed : {}'.format(line))
                else:
                    # Store index
                    data = index.groups()
                    image.add_index(
                        line=int(data[0]),
                        word=data[1].decode('utf-8'),
                        score=float(data[2]),
                        box=[
                            int(data[3]),
                            int(data[4]),
                            int(data[5]),
                            int(data[6]),
                        ],
                    )
