from django.db import models
from django.conf import settings
from elasticsearch import Elasticsearch
import hashlib
import json
import base64
import uuid

class Image(models.Model):
    """
    A document image
    """
    id = models.UUIDField(default=uuid.uuid4, primary_key=True)
    iiif_url = models.URLField(unique=True)

    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return '{} - {}'.format(self.id, self.iiif_url)

    def add_index(self, line, word, score, box):
        """
        Store an index in ElasticSearch
        """
        assert isinstance(line, int)
        assert isinstance(word, str)
        assert isinstance(score, float)
        assert isinstance(box, list)
        assert len(box) == 4
        assert all([isinstance(x, int) for x in box])

        # Build payload and its hash
        payload = {
            'image': self.id.hex,
            'word': word,
            'score': score,
            'box': box,
        }
        h = hashlib.md5()
        h.update(json.dumps(payload, sort_keys=True).encode('utf-8'))
        index_id = base64.urlsafe_b64encode(h.digest()).decode('utf-8')

        # Insert in ElasticSearch database
        # using the calculated index to avoid duplicates
        elastic = Elasticsearch(settings.ELASTIC_SEARCH_HOSTS)
        res = elastic.index(
            index='volume_xxx', # TODO: use volume ?
            doc_type='word',
            id=index_id,
            body=payload,
        )
        assert res['result'] in ('created', 'updated'), \
            'Failed to insert word index {} : {}'.format(index_id, res)

        return res['created']
